DROP SEQUENCE IF EXISTS messages_id_seq CASCADE;
DROP TRIGGER IF EXISTS insert_messages_trigger ON messages CASCADE;
DROP TABLE IF EXISTS messages CASCADE;

CREATE OR REPLACE FUNCTION message_table_name( tbl_date INTEGER )
RETURNS CHAR(30) AS $$
DECLARE
the_hour CHAR(2);
BEGIN
	the_hour := (select extract(hour from (select TIMESTAMP 'epoch' at time zone 'UTC' + tbl_date * INTERVAL '1 second')::timestamp))::text;
	IF (the_hour::int < 10) THEN
		the_hour := '0'||the_hour::text;
	END IF;

	RETURN 'messages_' || REPLACE(((select TIMESTAMP 'epoch' at time zone 'UTC' + tbl_date * INTERVAL '1 second')::DATE)::text, '-','_') || '_' || the_hour::text;
END;
$$
LANGUAGE plpgsql;

CREATE OR REPLACE FUNCTION create_table_function( tbl_date INTEGER )
RETURNS CHAR(30) AS $$
DECLARE
table_name CHAR(30);
_hour CHAR(2);
_starttime integer;
_endtime integer;
BEGIN
	table_name := message_table_name(tbl_date);
	IF EXISTS (
		SELECT *
		FROM   pg_catalog.pg_tables 
		WHERE    tablename  = table_name
		) THEN
		RETURN table_name;
	ELSE
		_hour := (select extract(hour from (select TIMESTAMP 'epoch' at time zone 'UTC' + tbl_date * INTERVAL '1 second')::timestamp))::text;
		_starttime := (select EXTRACT(EPOCH FROM (select (select TIMESTAMP 'epoch' at time zone 'UTC' + tbl_date * INTERVAL '1 second')::date::text || ' ' || _hour || ':00' )::timestamp));
		_endtime := (select _starttime + 3600);
		EXECUTE ( 'CREATE TABLE '
			|| quote_ident(table_name)
			|| ' ( CHECK ( rtime < '
			|| _endtime
			|| ' AND rtime >= '
			|| _starttime
			|| '), PRIMARY KEY (id, rtime)
		) INHERITS (messages);' );
		PERFORM partition_maintenance_function();
	END IF;
	RETURN table_name;
END;
$$
LANGUAGE plpgsql;


CREATE SEQUENCE messages_id_seq;
CREATE TABLE messages 
 (
    id BIGINT NOT NULL DEFAULT nextval('messages_id_seq'), 
    ip VARCHAR(110) NOT NULL DEFAULT '0',
    rtime INTEGER NOT NULL,
    facility INTEGER NOT NULL, 
    severity INTEGER NOT NULL, 
    mtime INTEGER NOT NULL,
    host VARCHAR(129) DEFAULT NULL, 
    ident VARCHAR(80) DEFAULT NULL, 
    pid VARCHAR(128), 
    event INT,
    msg TEXT NOT NULL, 
    fts TEXT DEFAULT '',
    PRIMARY KEY (id, rtime)
 );
CREATE INDEX messages_fts_idx ON messages USING gin( to_tsvector('english', fts) );
CREATE TRIGGER insert_messages_trigger
	BEFORE INSERT ON messages
	FOR EACH ROW EXECUTE PROCEDURE messages_insert_trigger();


--UPDATE db VERSION
UPDATE service SET version = 21;

