<?php
require_once (dirname(__FILE__) . '/base/baselogmessageset.php');
require_once('op5/mayi.php');

class LogMessageSet_InvalidColumnException extends Exception {}

/**
 * @todo : documentation
 */
class LogMessageSet_Model extends BaseLogMessageSet_Model {
	/**
	 * Return resource name of this object
	 * @return string
	 */
	public function mayi_resource() {
		return "monitor.logger.messages";
	}

	/**
	 * Since counting log messages is quite hard, don't do that, and return an
	 * empty totals list instead.
	 */
	public function get_totals() {
		return array();
	}

	protected function get_auth_filter() {
		$mayi = op5mayi::instance();

		$filter = new LivestatusFilterAnd();
		$filter->add(parent::get_auth_filter());

		/*
		 * FIXME: don't assume list view (:read.list suffix), implement when
		 * handling logger hosts more generic
		 */
		if (! $mayi->run( 'monitor.logger.messages:read.list', array (
			'monitor' => array (
				'logger' => array (
					'hosts' => 2
				)
			)
		) )) {
			/* Without license, only give access to localhost */
			$allowed_hosts_filter = new LivestatusFilterOr();
			$allowed_hosts_filter->add(new LivestatusFilterMatch('ip', '^127.[0-9]+.[0-9]+.[0-9]+$', '~'));
			$allowed_hosts_filter->add(new LivestatusFilterMatch('ip', '::1'));
			$filter->add($allowed_hosts_filter);
		}
		$auth = Auth::instance();
		if (!$auth->authorized_for('logger_access')) {
			throw new ORMException("You are not authorized to access Logger");
		}
		return $filter;
	}

	/*
	 * Generate an SQL query compatible with the logfilter binary representing
	 * this set.
	 */
	public function generate_logfilter_query($columns, $tablename = 'stdin') {
		/* Do some javascript-like magic so we can use function()use($self){} */
		$self = $this;

		try {
			/* Generate filter */
			$where = $this->filter->visit(
				new LogfilterSQLBuilderVisitor(
					function ($name) use($self) {
						$colname = LogMessagePool_Model::map_name_to_backend($name, '');
						if ($colname === false) {
							throw new LogMessageSet_InvalidColumnException();
						}
						return $colname;
					}), 0);

			/* Combine to a query */
			return "SELECT " . implode(',', $columns) . " FROM " . $tablename .
				 " WHERE " . $where;
		} catch (LogMessageSet_InvalidColumnException $e) {
			/*
			 * This is a user error, we should in the future handle an error
			 * message here... No user should ever see it when when using the
			 * gui, since this case is filetered earlier anyway.
			 */
			return false;
		}
	}
}
